__title__ = 'organize-tool'
__description__ = 'The file management automation tool.'
__url__ = 'http://github.com/tfeldmann/organize'
__version__ = '1.2'
__author__ = 'Thomas Feldmann'
__author_email__ = 'mail@tfeldmann.de'
__license__ = 'MIT'
__copyright__ = 'Copyright (c)Thomas Feldmann'
